@@ -1,10 +1,12 @@
-from io import BufferedReader
+from io import BufferedReader, BytesIO
 from typing import Any, Union
+from lxml import html
 import requests
 import logging
 import backoff
 import websocket
 import json
+import zipfile
 
 logger = logging.getLogger(__name__)
 
@@ -14,17 +16,46 @@ class EscriptoriumConnector:
         self,
         base_url: str,
         api_url: str,
-        token: str,
-        cookie: str = None,
+        username: str,
+        password: str,
         project: str = None,
     ):
         # Make sure the urls terminates with a front slash
         self.api_url = api_url if api_url[-1] == "/" else api_url + "/"
         self.base_url = base_url if base_url[-1] == "/" else base_url + "/"
 
-        self.headers = {"Accept": "application/json", "Authorization": f"Token {token}"}
+        self.headers = {"Accept": "application/json"}
         self.project = project
-        self.cookie = cookie
+
+        self.session_requests = requests.session()
+        login_url = f"""{self.base_url}login/"""
+        result = self.session_requests.get(login_url)
+        tree = html.fromstring(result.text)
+        authenticity_token = list(
+            set(tree.xpath("//input[@name='csrfmiddlewaretoken']/@value"))
+        )[0]
+        payload = {
+            "username": username,
+            "password": password,
+            "csrfmiddlewaretoken": authenticity_token,
+        }
+        result = self.session_requests.post(
+            login_url, data=payload, headers={**self.headers, "referer": login_url}
+        )
+        self.cookie = "; ".join(
+            [
+                f"""{k}={v}"""
+                for k, v in self.session_requests.cookies.get_dict().items()
+            ]
+        )
+        result = self.session_requests.get(
+            self.base_url + "profile/apikey/", headers=self.headers
+        )
+        tree = html.fromstring(result.text)
+        api_key = list(set(tree.xpath("//button[@id='api-key-clipboard']/@data-key")))[
+            0
+        ]
+        self.headers["Authorization"] = f"""Token {api_key}"""
 
     def __on_message(self, ws, message):
         logging.debug(message)
@@ -195,7 +226,7 @@ class EscriptoriumConnector:
         document_pk: int,
         part_pk: int,
         transcription_pk: int,
-    ) -> Union[bytes, None]:
+    ) -> Union[list[bytes], None]:
         return self.__download_part_output_transcription(
             document_pk, part_pk, transcription_pk, "alto"
         )
@@ -205,7 +236,7 @@ class EscriptoriumConnector:
         document_pk: int,
         part_pk: int,
         transcription_pk: int,
-    ) -> Union[bytes, None]:
+    ) -> Union[list[bytes], None]:
         return self.__download_part_output_transcription(
             document_pk, part_pk, transcription_pk, "pagexml"
         )
@@ -215,7 +246,7 @@ class EscriptoriumConnector:
         document_pk: int,
         part_pk: int,
         transcription_pk: int,
-    ) -> Union[bytes, None]:
+    ) -> Union[list[bytes], None]:
         return self.__download_part_output_transcription(
             document_pk, part_pk, transcription_pk, "text"
         )
@@ -226,7 +257,7 @@ class EscriptoriumConnector:
         part_pk: int,
         transcription_pk: int,
         output_type: str,
-    ) -> Union[bytes, None]:
+    ) -> Union[list[bytes], None]:
         if self.cookie is None:
             raise Exception("Must use websockets to download ALTO exports")
 
@@ -260,13 +291,13 @@ class EscriptoriumConnector:
                 f"Did not receive a link to download ALTO export for {document_pk}, {part_pk}, {transcription_pk}"
             )
             return None
-
         alto_request = self.__get_url(f"{self.base_url}{download_link}")
 
         if alto_request.status_code != 200:
             return None
 
-        return alto_request.content
+        zip = zipfile.ZipFile(BytesIO(alto_request.content))
+        return [zip.read(x) for x in zip.infolist()]
 
     def upload_part_transcription(
         self,
